@@ -13,9 +13,9 @@
 """Cacheable object and cache management base classes.
 """
 
-from logging import getLogger
 import sys
 import time
+from logging import getLogger
 
 from AccessControl.class_init import InitializeClass
 from AccessControl.Permissions import view_management_screens
@@ -29,6 +29,7 @@ from Acquisition import aq_inner
 from Acquisition import aq_parent
 from App.special_dtml import DTMLFile
 
+
 ZCM_MANAGERS = '__ZCacheManager_ids__'
 
 ViewManagementScreensPermission = view_management_screens
@@ -39,30 +40,32 @@ LOG = getLogger('Cache')
 def isCacheable(ob):
     return getattr(aq_base(ob), '_isCacheable', 0)
 
+
 def managersExist(ob):
     # Returns 1 if any CacheManagers exist in the context of ob.
     if aq_get(ob, ZCM_MANAGERS, None, 1):
         return 1
     return 0
 
+
 def filterCacheTab(ob):
     return managersExist(ob)
 
+
 def filterCacheManagers(orig, container, name, value, extra):
-    '''
+    """
     This is a filter method for aq_acquire.
     It causes objects to be found only if they are
     in the list of cache managers.
-    '''
-    if (hasattr(aq_base(container), ZCM_MANAGERS) and
-        name in getattr(container, ZCM_MANAGERS)):
+    """
+    if hasattr(aq_base(container), ZCM_MANAGERS) and \
+       name in getattr(container, ZCM_MANAGERS):
         return 1
     return 0
 
+
 def getVerifiedManagerIds(container):
-    '''
-    Gets the list of cache managers in a container, verifying each one.
-    '''
+    """Gets the list of cache managers in a container, verifying each one."""
     ids = getattr(container, ZCM_MANAGERS, ())
     rval = []
     for id in ids:
@@ -77,46 +80,51 @@ def getVerifiedManagerIds(container):
 manager_timestamp = 0
 
 
-class Cacheable:
-    '''Mix-in for cacheable objects.
-    '''
+class Cacheable(object):
+    """Mix-in for cacheable objects."""
 
-    manage_options = ({
-        'label':'Cache',
-        'action':'ZCacheable_manage',
-        'filter':filterCacheTab,
-        },)
+    manage_options = (
+        {
+            'label': 'Cache',
+            'action': 'ZCacheable_manage',
+            'filter': filterCacheTab,
+        },
+    )
 
     security = ClassSecurityInfo()
     security.setPermissionDefault(ChangeCacheSettingsPermission, ('Manager',))
 
-    security.declareProtected(ViewManagementScreensPermission,
-                              'ZCacheable_manage')
+    security.declareProtected(ViewManagementScreensPermission, 'ZCacheable_manage')  # NOQA: D001,E501
     ZCacheable_manage = DTMLFile('dtml/cacheable', globals())
 
     _v_ZCacheable_cache = None
     _v_ZCacheable_manager_timestamp = 0
     __manager_id = None
-    __enabled = 1
-    _isCacheable = 1
+    __enabled = True
+    _isCacheable = True
 
-    security.declarePrivate('ZCacheable_getManager')
+    @security.private
     def ZCacheable_getManager(self):
-        '''Returns the currently associated cache manager.'''
+        """Returns the currently associated cache manager."""
         manager_id = self.__manager_id
         if manager_id is None:
             return None
         try:
             return aq_acquire(
-                self, manager_id, containment=1,
-                filter=filterCacheManagers, extra=None, default=None)
+                self,
+                manager_id,
+                containment=1,
+                filter=filterCacheManagers,
+                extra=None,
+                default=None
+            )
         except AttributeError:
             return None
 
-    security.declarePrivate('ZCacheable_getCache')
+    @security.private
     def ZCacheable_getCache(self):
-        '''Gets the cache associated with this object.
-        '''
+        """Gets the cache associated with this object.
+        """
         if self.__manager_id is None:
             return None
         c = self._v_ZCacheable_cache
@@ -134,26 +142,31 @@ class Cacheable:
         self._v_ZCacheable_manager_timestamp = manager_timestamp
         return c
 
-    security.declarePrivate('ZCacheable_isCachingEnabled')
+    @security.private
     def ZCacheable_isCachingEnabled(self):
-        '''
+        """
         Returns true only if associated with a cache manager and
         caching of this method is enabled.
-        '''
+        """
         return self.__enabled and self.ZCacheable_getCache()
 
-    security.declarePrivate('ZCacheable_getObAndView')
+    @security.private
     def ZCacheable_getObAndView(self, view_name):
         # Returns self and view_name unchanged.
         return self, view_name
 
-    security.declarePrivate('ZCacheable_get')
-    def ZCacheable_get(self, view_name='', keywords=None,
-                       mtime_func=None, default=None):
-        '''Retrieves the cached view for the object under the
+    @security.private
+    def ZCacheable_get(
+        self,
+        view_name='',
+        keywords=None,
+        mtime_func=None,
+        default=None
+    ):
+        """Retrieves the cached view for the object under the
         conditions specified by keywords. If the value is
         not yet cached, returns the default.
-        '''
+        """
         c = self.ZCacheable_getCache()
         if c is not None and self.__enabled:
             ob, view_name = self.ZCacheable_getObAndView(view_name)
@@ -161,35 +174,37 @@ class Cacheable:
                 val = c.ZCache_get(ob, view_name, keywords,
                                    mtime_func, default)
                 return val
-            except:
-                LOG.warn('ZCache_get() exception',
-                    exc_info=sys.exc_info())
+            except Exception:
+                LOG.warning('ZCache_get() exception')
                 return default
         return default
 
-    security.declarePrivate('ZCacheable_set')
-    def ZCacheable_set(self, data, view_name='', keywords=None,
-                       mtime_func=None):
-        '''Cacheable views should call this method after generating
+    @security.private
+    def ZCacheable_set(
+        self,
+        data,
+        view_name='',
+        keywords=None,
+        mtime_func=None
+    ):
+        """Cacheable views should call this method after generating
         cacheable results. The data argument can be of any Python type.
-        '''
+        """
         c = self.ZCacheable_getCache()
         if c is not None and self.__enabled:
             ob, view_name = self.ZCacheable_getObAndView(view_name)
             try:
                 c.ZCache_set(ob, data, view_name, keywords,
                              mtime_func)
-            except:
-                LOG.warn('ZCache_set() exception',
-                    exc_info=sys.exc_info())
+            except Exception:
+                LOG.warning('ZCache_set() exception')
 
-    security.declareProtected(ViewManagementScreensPermission,
-                              'ZCacheable_invalidate')
+    @security.protected(ViewManagementScreensPermission)
     def ZCacheable_invalidate(self, view_name='', REQUEST=None):
-        '''Called after a cacheable object is edited. Causes all
+        """Called after a cacheable object is edited. Causes all
         cache entries that apply to the view_name to be removed.
         Returns a status message.
-        '''
+        """
         c = self.ZCacheable_getCache()
         if c is not None:
             ob, view_name = self.ZCacheable_getObAndView(view_name)
@@ -197,26 +212,25 @@ class Cacheable:
                 message = c.ZCache_invalidate(ob)
                 if not message:
                     message = 'Invalidated.'
-            except:
+            except Exception:
                 exc = sys.exc_info()
                 try:
-                    LOG.warn('ZCache_invalidate() exception',
-                        exc_info=exc)
+                    LOG.warning('ZCache_invalidate() exception')
                     message = 'An exception occurred: %s: %s' % exc[:2]
                 finally:
                     exc = None
         else:
             message = 'This object is not associated with a cache manager.'
+
         if REQUEST is not None:
             return self.ZCacheable_manage(
                 self, REQUEST, management_view='Cache',
                 manage_tabs_message=message)
-        else:
-            return message
+        return message
 
-    security.declarePrivate('ZCacheable_getModTime')
+    @security.private
     def ZCacheable_getModTime(self, mtime_func=None):
-        '''Returns the highest of the last mod times.'''
+        """Returns the highest of the last mod times."""
         # Based on:
         #   mtime_func
         #   self.mtime
@@ -226,32 +240,31 @@ class Cacheable:
             # Allow mtime_func to influence the mod time.
             mtime = mtime_func()
         base = aq_base(self)
-        mtime = max(getattr(base, '_p_mtime', mtime), mtime)
+        mtime = max(getattr(base, '_p_mtime', mtime) or 0, mtime)
         klass = getattr(base, '__class__', None)
         if klass:
-            mtime = max(getattr(klass, '_p_mtime', mtime), mtime)
+            klass_mtime = getattr(klass, '_p_mtime', mtime)
+            if isinstance(klass_mtime, int):
+                mtime = max(klass_mtime, mtime)
         return mtime
 
-    security.declareProtected(ViewManagementScreensPermission,
-                              'ZCacheable_getManagerId')
+    @security.protected(ViewManagementScreensPermission)
     def ZCacheable_getManagerId(self):
-        '''Returns the id of the current ZCacheManager.'''
+        """Returns the id of the current ZCacheManager."""
         return self.__manager_id
 
-    security.declareProtected(ViewManagementScreensPermission,
-                              'ZCacheable_getManagerURL')
+    @security.protected(ViewManagementScreensPermission)
     def ZCacheable_getManagerURL(self):
-        '''Returns the URL of the current ZCacheManager.'''
+        """Returns the URL of the current ZCacheManager."""
         manager = self.ZCacheable_getManager()
         if manager is not None:
             return manager.absolute_url()
         return None
 
-    security.declareProtected(ViewManagementScreensPermission,
-                              'ZCacheable_getManagerIds')
+    @security.protected(ViewManagementScreensPermission)
     def ZCacheable_getManagerIds(self):
-        '''Returns a list of mappings containing the id and title
-        of the available ZCacheManagers.'''
+        """Returns a list of mappings containing the id and title
+        of the available ZCacheManagers."""
         rval = []
         ob = self
         used_ids = {}
@@ -262,18 +275,16 @@ class Cacheable:
                     manager = getattr(ob, id, None)
                     if manager is not None:
                         id = manager.getId()
-                        if not used_ids.has_key(id):
-                            title = getattr(aq_base(manager),
-                                            'title', '')
-                            rval.append({'id':id, 'title':title})
+                        if id not in used_ids:
+                            title = getattr(aq_base(manager), 'title', '')
+                            rval.append({'id': id, 'title': title})
                             used_ids[id] = 1
             ob = aq_parent(aq_inner(ob))
         return tuple(rval)
 
-    security.declareProtected(ChangeCacheSettingsPermission,
-                              'ZCacheable_setManagerId')
+    @security.protected(ChangeCacheSettingsPermission)
     def ZCacheable_setManagerId(self, manager_id, REQUEST=None):
-        '''Changes the manager_id for this object.'''
+        """Changes the manager_id for this object."""
         self.ZCacheable_invalidate()
         if not manager_id:
             # User requested disassociation
@@ -283,46 +294,54 @@ class Cacheable:
             manager_id = str(manager_id)
         self.__manager_id = manager_id
         self._v_ZCacheable_cache = None
+
         if REQUEST is not None:
             return self.ZCacheable_manage(
-                self, REQUEST, management_view='Cache',
-                manage_tabs_message='Cache settings changed.')
+                self,
+                REQUEST,
+                management_view='Cache',
+                manage_tabs_message='Cache settings changed.'
+            )
 
-    security.declareProtected(ViewManagementScreensPermission,
-                              'ZCacheable_enabled')
+    @security.protected(ViewManagementScreensPermission)
     def ZCacheable_enabled(self):
-        '''Returns true if caching is enabled for this object
-        or method.'''
+        """Returns true if caching is enabled for this object or method."""
         return self.__enabled
 
-    security.declareProtected(ChangeCacheSettingsPermission,
-                              'ZCacheable_setEnabled')
+    @security.protected(ChangeCacheSettingsPermission)
     def ZCacheable_setEnabled(self, enabled=0, REQUEST=None):
-        '''Changes the enabled flag.'''
+        """Changes the enabled flag."""
         self.__enabled = enabled and 1 or 0
+
         if REQUEST is not None:
             return self.ZCacheable_manage(
                 self, REQUEST, management_view='Cache',
                 manage_tabs_message='Cache settings changed.')
 
-    security.declareProtected(ViewManagementScreensPermission,
-                              'ZCacheable_configHTML')
+    @security.protected(ViewManagementScreensPermission)
     def ZCacheable_configHTML(self):
-        '''Override to provide configuration of caching
+        """Override to provide configuration of caching
         behavior that can only be specific to the cacheable object.
-        '''
+        """
         return ''
 
 
 InitializeClass(Cacheable)
 
 
-def findCacheables(ob, manager_id, require_assoc, subfolders,
-                   meta_types, rval, path):
-    '''
+def findCacheables(
+    ob,
+    manager_id,
+    require_assoc,
+    subfolders,
+    meta_types,
+    rval,
+    path
+):
+    """
     Used by the CacheManager UI.  Recursive.  Similar to the Zope
     "Find" function.  Finds all Cacheable objects in a hierarchy.
-    '''
+    """
     try:
         if meta_types:
             subobs = ob.objectValues(meta_types)
@@ -340,13 +359,13 @@ def findCacheables(ob, manager_id, require_assoc, subfolders,
             if not sm.checkPermission('Change cache settings', subob):
                 continue
             subpath = path + (subob.getId(),)
-            icon = getattr(aq_base(subob), 'icon', '')
             info = {
                 'sortkey': subpath,
                 'path': '/'.join(subpath),
                 'title': getattr(aq_base(subob), 'title', ''),
-                'icon': icon,
-                'associated': associated,}
+                'icon': None,
+                'associated': associated,
+            }
             rval.append(info)
 
         # Visit subfolders.
@@ -357,22 +376,22 @@ def findCacheables(ob, manager_id, require_assoc, subfolders,
                 subpath = path + (subob.getId(),)
                 if hasattr(aq_base(subob), 'objectValues'):
                     if sm.checkPermission(
-                        'Access contents information', subob):
+                            'Access contents information', subob):
                         findCacheables(
                             subob, manager_id, require_assoc,
                             subfolders, meta_types, rval, subpath)
-    except:
+    except Exception:
         # Ignore exceptions.
         import traceback
         traceback.print_exc()
 
 
-class Cache:
-    '''
+class Cache(object):
+    """
     A base class (and interface description) for caches.
     Note that Cache objects are not intended to be visible by
     restricted code.
-    '''
+    """
 
     def ZCache_invalidate(self, ob):
         raise NotImplementedError
@@ -401,25 +420,26 @@ class Cache:
         raise NotImplementedError
 
 
-class CacheManager:
-    '''
+class CacheManager(object):
+    """
     A base class for cache managers.  Implement ZCacheManager_getCache().
-    '''
+    """
 
     security = ClassSecurityInfo()
     security.setPermissionDefault(ChangeCacheSettingsPermission, ('Manager',))
 
-    security.declarePrivate('ZCacheManager_getCache')
+    @security.private
     def ZCacheManager_getCache(self):
         raise NotImplementedError
 
     _isCacheManager = 1
 
     manage_options = (
-        {'label':'Associate',
-         'action':'ZCacheManager_associate',
-         },
-        )
+        {
+            'label': 'Associate',
+            'action': 'ZCacheManager_associate',
+        },
+    )
 
     def manage_afterAdd(self, item, container):
         # Adds self to the list of cache managers in the container.
@@ -445,37 +465,50 @@ class CacheManager:
                 global manager_timestamp
                 manager_timestamp = time.time()
 
-    security.declareProtected(ChangeCacheSettingsPermission,
-                              'ZCacheManager_associate')
+    security.declareProtected(ChangeCacheSettingsPermission, 'ZCacheManager_associate')  # NOQA: D001,E501
     ZCacheManager_associate = DTMLFile('dtml/cmassoc', globals())
 
-    security.declareProtected(ChangeCacheSettingsPermission,
-                              'ZCacheManager_locate')
-    def ZCacheManager_locate(self, require_assoc, subfolders,
-                             meta_types=[], REQUEST=None):
-        '''Locates cacheable objects.
-        '''
+    @security.protected(ChangeCacheSettingsPermission)
+    def ZCacheManager_locate(
+        self,
+        require_assoc,
+        subfolders,
+        meta_types=[],
+        REQUEST=None
+    ):
+        """Locates cacheable objects.
+        """
         ob = aq_parent(aq_inner(self))
         rval = []
         manager_id = self.getId()
         if '' in meta_types:
             # User selected "All".
             meta_types = []
-        findCacheables(ob, manager_id, require_assoc, subfolders,
-                       meta_types, rval, ())
+        findCacheables(
+            ob,
+            manager_id,
+            require_assoc,
+            subfolders,
+            meta_types,
+            rval,
+            ()
+        )
+
         if REQUEST is not None:
             return self.ZCacheManager_associate(
-                self, REQUEST, show_results=1, results=rval,
-                management_view="Associate")
-        else:
-            return rval
+                self,
+                REQUEST,
+                show_results=1,
+                results=rval,
+                management_view="Associate"
+            )
+        return rval
 
-    security.declareProtected(ChangeCacheSettingsPermission,
-                              'ZCacheManager_setAssociations')
+    @security.protected(ChangeCacheSettingsPermission)
     def ZCacheManager_setAssociations(self, props=None, REQUEST=None):
-        '''Associates and un-associates cacheable objects with this
+        """Associates and un-associates cacheable objects with this
         cache manager.
-        '''
+        """
         addcount = 0
         remcount = 0
         parent = aq_parent(aq_inner(self))
@@ -501,11 +534,13 @@ class CacheManager:
                     if manager_id == my_id:
                         ob.ZCacheable_setManagerId(None)
                         remcount = remcount + 1
+
         if REQUEST is not None:
             return self.ZCacheManager_associate(
                 self, REQUEST, management_view="Associate",
-                manage_tabs_message='%d association(s) made, %d removed.'%
+                manage_tabs_message='%d association(s) made, %d removed.' %
                 (addcount, remcount)
-                )
+            )
+
 
 InitializeClass(CacheManager)
