@@ -13,6 +13,7 @@
 """DTML Method objects.
 """
 import re
+from warnings import warn
 
 from six import PY2
 from six import PY3
@@ -37,6 +38,8 @@ from DocumentTemplate.permissions import change_dtml_methods
 from DocumentTemplate.security import RestrictedDTML
 from OFS import bbb
 from OFS.Cache import Cacheable
+from OFS.History import Historical
+from OFS.History import html_diff
 from OFS.role import RoleManager
 from OFS.SimpleItem import Item_w__name__
 from OFS.SimpleItem import PathReprProvider
@@ -63,6 +66,7 @@ class DTMLMethod(
     Implicit,
     RoleManager,
     Item_w__name__,
+    Historical,
     Cacheable
 ):
     """ DocumentTemplate.HTML objects that act as methods of their containers.
@@ -95,11 +99,20 @@ class DTMLMethod(
             'label': 'Proxy',
             'action': 'manage_proxyForm',
         },
-    ) + RoleManager.manage_options
+    ) + Historical.manage_options
+      + RoleManager.manage_options
       + Item_w__name__.manage_options
       + Cacheable.manage_options
     )
 
+    # Careful in permission changes--used by DTMLDocument!
+    security.declareProtected(change_dtml_methods,  # NOQA: D001
+                              'manage_historyCopy')
+    security.declareProtected(change_dtml_methods,  # NOQA: D001
+                              'manage_beforeHistoryCopy')
+    security.declareProtected(change_dtml_methods,  # NOQA: D001
+                              'manage_afterHistoryCopy')
+
     # More reasonable default for content-type for http HEAD requests.
     default_content_type = 'text/html'
 
@@ -365,25 +378,33 @@ class DTMLMethod(
             RESPONSE.setHeader('Content-Type', 'text/plain')
         return self.read()
 
-    if bbb.HAS_ZSERVER:
+    @security.protected(change_dtml_methods)
+    def PUT(self, REQUEST, RESPONSE):
+        """ Handle HTTP PUT requests.
+        """
+        self.dav__init(REQUEST, RESPONSE)
+        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
+        body = safe_file_data(REQUEST.get('BODY', ''))
+        self._validateProxy(REQUEST)
+        self.munge(body)
+        self.ZCacheable_invalidate()
+        RESPONSE.setStatus(204)
+        return RESPONSE
 
-        @security.protected(change_dtml_methods)
-        def PUT(self, REQUEST, RESPONSE):
-            """ Handle FTP / HTTP PUT requests.
-            """
-            self.dav__init(REQUEST, RESPONSE)
-            self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
-            body = REQUEST.get('BODY', '')
-            self._validateProxy(REQUEST)
-            self.munge(body)
-            self.ZCacheable_invalidate()
-            RESPONSE.setStatus(204)
-            return RESPONSE
+    def manage_historyCompare(self, rev1, rev2, REQUEST,
+                              historyComparisonResults=''):
+        return DTMLMethod.inheritedAttribute('manage_historyCompare')(
+            self, rev1, rev2, REQUEST,
+            historyComparisonResults=html_diff(rev1.read(), rev2.read()))
+
+    if bbb.HAS_ZSERVER:
 
         @security.protected(ftp_access)
         def manage_FTPget(self):
             """ Get source for FTP download.
             """
+            warn(u'manage_FTPget is deprecated and will be removed in Zope 5.',
+                 DeprecationWarning, stacklevel=2)
             return self.read()
 
 
