@@ -259,6 +259,27 @@ class FileTests(unittest.TestCase):
         self.assertEqual(resp.getStatus(), 200)
         self.assertEqual(data, bytes(self.file.data))
 
+    def testPUT(self):
+        s = b'# some python\n'
+
+        # with content type
+        data = BytesIO(s)
+        req = aputrequest(data, 'text/x-python')
+        req.processInputs()
+        self.file.PUT(req, req.RESPONSE)
+
+        self.assertEqual(self.file.content_type, 'text/x-python')
+        self.assertEqual(self.file.data, s)
+
+        # without content type
+        data.seek(0)
+        req = aputrequest(data, '')
+        req.processInputs()
+        self.file.PUT(req, req.RESPONSE)
+
+        self.assertEqual(self.file.content_type, 'text/x-python')
+        self.assertEqual(self.file.data, s)
+
     def testIndexHtmlWithPdata(self):
         self.file.manage_upload(b'a' * (2 << 16))  # 128K
         self.file.index_html(self.app.REQUEST, self.app.REQUEST.RESPONSE)
@@ -283,6 +304,15 @@ class FileTests(unittest.TestCase):
                                        b'come to the aid of the Party.')
         self.assertTrue(b'Party' in self.file.PrincipiaSearchSource())
 
+    def test_manage_DAVget_binary(self):
+        self.assertEqual(self.file.manage_DAVget(), self.data)
+
+    def test_manage_DAVget_text(self):
+        text = (b'Now is the time for all good men to '
+                b'come to the aid of the Party.')
+        self.file.manage_edit('foobar', 'text/plain', filedata=text)
+        self.assertEqual(self.file.manage_DAVget(), text)
+
     def test_interfaces(self):
         from zope.interface.verify import verifyClass
         from OFS.Image import File
@@ -298,6 +328,16 @@ class FileTests(unittest.TestCase):
         self.assertRaises(TypeError, self.file.update_data,
                           data=val, content_type='text/plain')
 
+    def testStr(self):
+        small_data = b'small data'
+        self.file.manage_upload(file=small_data)
+        self.assertEqual(str(self.file), small_data.decode())
+
+        # Make sure Pdata contents are handled correctly
+        big_data = b'a' * (2 << 16)
+        self.file.manage_upload(file=big_data)
+        self.assertEqual(str(self.file), big_data.decode())
+
 
 class ImageTests(FileTests):
     content_type = 'image/gif'
@@ -321,6 +361,8 @@ class ImageTests(FileTests):
         self.file.manage_changeProperties(alt='bar')
         self.assertEqual(self.file.tag(), (tag_fmt % ('bar', 'foo')))
 
+    testStr = testTag
+
     def testViewImageOrFile(self):
         request = self.app.REQUEST
         response = request.RESPONSE
