@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2004, 2005 Zope Corporation and Contributors.
+# Copyright (c) 2004, 2005 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,16 +12,16 @@
 #
 ##############################################################################
 """Size adapters for testing
-
-$Id: test_size.py 99527 2009-04-26 16:33:20Z hannosch $
 """
+from __future__ import absolute_import
 
-from zope.interface import implements
+from zope.interface import implementer
 from zope.size.interfaces import ISized
 
+
+@implementer(ISized)
 class SimpleContentSize(object):
     """Size for ``SimpleContent`` objects."""
-    implements(ISized)
 
     def __init__(self, context):
         self.context = context
@@ -32,9 +32,10 @@ class SimpleContentSize(object):
     def sizeForDisplay(self):
         return "What is the meaning of life?"
 
+
+@implementer(ISized)
 class FancyContentSize(object):
     """Size for ``SimpleContent`` objects."""
-    implements(ISized)
 
     def __init__(self, context):
         self.context = context
@@ -45,6 +46,7 @@ class FancyContentSize(object):
     def sizeForDisplay(self):
         return "That's not the meaning of life!"
 
+
 def test_size():
     """
     Test size adapters
@@ -57,8 +59,10 @@ def test_size():
       >>> configure_zcml = '''
       ... <configure xmlns="http://namespaces.zope.org/zope"
       ...            xmlns:five="http://namespaces.zope.org/five">
-      ...   <five:sizable class="Products.Five.tests.testing.simplecontent.SimpleContent" />
-      ...   <five:sizable class="Products.Five.tests.testing.fancycontent.FancyContent" />
+      ...   <five:sizable
+      ...    class="Products.Five.tests.testing.simplecontent.SimpleContent" />
+      ...   <five:sizable
+      ...    class="Products.Five.tests.testing.fancycontent.FancyContent" />
       ...   <adapter
       ...       for="Products.Five.tests.testing.simplecontent.ISimpleContent"
       ...       provides="zope.size.interfaces.ISized"
@@ -72,23 +76,26 @@ def test_size():
       ... </configure>'''
 
       >>> import Products.Five
-      >>> from Products.Five import zcml
+      >>> from Zope2.App import zcml
       >>> zcml.load_config('meta.zcml', Products.Five)
       >>> zcml.load_string(configure_zcml)
+      >>> folder = self.folder  # NOQA: F821
 
-      >>> from Products.Five.tests.testing.simplecontent import manage_addSimpleContent
-      >>> from Products.Five.tests.testing.fancycontent import manage_addFancyContent
+      >>> from Products.Five.tests.testing.simplecontent import (
+      ... manage_addSimpleContent)
+      >>> from Products.Five.tests.testing.fancycontent import (
+      ... manage_addFancyContent)
 
     We have registered an ``ISized`` adapter for SimpleContent:
 
-      >>> n = manage_addSimpleContent(self.folder, 'simple', 'Simple')
-      >>> self.folder.simple.get_size()
+      >>> n = manage_addSimpleContent(folder, 'simple', 'Simple')
+      >>> folder.simple.get_size()
       42
 
     Fancy content already has a ``get_size`` method
 
-      >>> n = manage_addFancyContent(self.folder, 'fancy', 'Fancy')
-      >>> self.folder.fancy.get_size()
+      >>> n = manage_addFancyContent(folder, 'fancy', 'Fancy')
+      >>> folder.fancy.get_size()
       43
 
     Clean up:
@@ -96,6 +103,7 @@ def test_size():
       >>> tearDown()
     """
 
+
 def test_suite():
     from Testing.ZopeTestCase import ZopeDocTestSuite
     return ZopeDocTestSuite()
