@@ -1,9 +1,13 @@
+"""``chameleon.tales`` expressions."""
+
 from ast import NodeTransformer
 from ast import parse
 
 from chameleon.astutil import Static
 from chameleon.astutil import Symbol
 from chameleon.codegen import template
+from chameleon.tales import NotExpr
+from chameleon.tales import StringExpr
 from six import class_types
 
 from AccessControl.ZopeGuards import guarded_apply
@@ -12,15 +16,19 @@ from AccessControl.ZopeGuards import guarded_getitem
 from AccessControl.ZopeGuards import guarded_iter
 from AccessControl.ZopeGuards import protected_inplacevar
 from OFS.interfaces import ITraversable
-from Products.PageTemplates.Expressions import render
 from RestrictedPython import RestrictingNodeTransformer
 from RestrictedPython.Utilities import utility_builtins
 from z3c.pt import expressions
 from zExceptions import NotFound
 from zExceptions import Unauthorized
+from zope.interface import implementer
+from zope.tales.tales import ExpressionEngine
 from zope.traversing.adapters import traversePathElement
 from zope.traversing.interfaces import TraversalError
 
+from .Expressions import render
+from .interfaces import IZopeAwareEngine
+
 
 _marker = object()
 
@@ -49,21 +57,16 @@ class BoboAwareZopeTraverse(object):
     def traverse(cls, base, request, path_items):
         """See ``zope.app.pagetemplate.engine``."""
 
-        length = len(path_items)
-        if length:
-            i = 0
-            method = cls.traverse_method
-            while i < length:
-                name = path_items[i]
-                i += 1
+        path_items = list(path_items)
+        path_items.reverse()
 
-                if ITraversable.providedBy(base):
-                    traverser = getattr(base, method)
-                    base = traverser(name)
-                else:
-                    base = traversePathElement(
-                        base, name, path_items[i:], request=request
-                    )
+        while path_items:
+            name = path_items.pop()
+            if ITraversable.providedBy(base):
+                base = getattr(base, cls.traverseMethod)(name)
+            else:
+                base = traversePathElement(base, name, path_items,
+                                           request=request)
 
         return base
 
@@ -154,14 +157,6 @@ class UntrustedPythonExpr(expressions.PythonExpr):
         (name, static(builtin)) for (name, builtin) in utility_builtins.items()
     ))
 
-    def rewrite(self, node):
-        if node.id == 'repeat':
-            node.id = 'wrapped_repeat'
-        else:
-            node = super(UntrustedPythonExpr, self).rewrite(node)
-
-        return node
-
     def parse(self, string):
         encoded = string.encode('utf-8')
         node = parse(encoded, mode='eval')
@@ -173,3 +168,67 @@ class UntrustedPythonExpr(expressions.PythonExpr):
         self.page_templates_expression_transformer.visit(node)
 
         return node
+
+
+# Whether an engine is Zope aware does not depend on the class
+# but how it is configured - especially, that is uses a Zope aware
+# `PathExpr` implementation.
+# Nevertheless, we mark the class as "Zope aware" for simplicity
+# assuming that users of the class use a proper `PathExpr`
+@implementer(IZopeAwareEngine)
+class ChameleonEngine(ExpressionEngine):
+    """Expression engine for ``chameleon.tales``.
+
+    Only partially implemented: its ``compile`` is currently unusable
+    """
+    def compile(self, expression):
+        raise NotImplementedError()
+
+
+types = dict(
+    python=UntrustedPythonExpr,
+    string=StringExpr,
+    not_=NotExpr,
+    exists=ExistsExpr,
+    path=PathExpr,
+    provider=expressions.ProviderExpr,
+    nocall=NocallExpr)
+
+
+def createChameleonEngine(types=types, untrusted=True, **overrides):
+    e = ChameleonEngine()
+
+    def norm(k):
+        return k[:-1] if k.endswith("_") else k
+
+    e.untrusted = untrusted
+    ts = e.types
+    for k, v in types.items():
+        k = norm(k)
+        e.registerType(k, v)
+    for k, v in overrides.items():
+        k = norm(k)
+        if k in ts:
+            del ts[k]
+        e.registerType(k, v)
+    return e
+
+
+def createTrustedChameleonEngine(**overrides):
+    ovr = dict(python=expressions.PythonExpr, path=TrustedPathExpr)
+    ovr.update(overrides)
+    return createChameleonEngine(untrusted=False, **ovr)
+
+
+_engine = createChameleonEngine()
+
+
+def getEngine():
+    return _engine
+
+
+_trusted_engine = createTrustedChameleonEngine()
+
+
+def getTrustedEngine():
+    return _trusted_engine
